@@ -20,18 +20,21 @@ public:
 	
 	const netcode::RawPacket* Pack() const;
 	
-	void SetSetup(const std::string& newSetup);
+	void SetSetupText(const std::string& newSetup);
 	void SetMapChecksum(const unsigned checksum);
 	void SetModChecksum(const unsigned checksum);
 	void SetRandomSeed(const unsigned seed);
 	
-	const std::string& GetSetup() const { return setupText; }
+	const std::string& GetSetupText() const { return setupText; }
 	unsigned GetMapChecksum() const { return mapChecksum; }
 	unsigned GetModChecksum() const { return modChecksum; }
 	unsigned GetRandomSeed() const { return randomSeed; }
 
 private:
+	// same as GameSetup::gameSetupText, but after any
+	// possible modifications by PreGame or GameServer
 	std::string setupText;
+
 	mutable std::vector<boost::uint8_t> compressed;
 
 	unsigned mapChecksum;
