@@ -1,10 +1,10 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include <map>
-#include <SDL_keycode.h>
-#include <set>
 #include <cfloat>
 
+#include <SDL_keycode.h>
+
 #include "PreGame.h"
 
 #include "ClientSetup.h"
@@ -34,7 +34,7 @@
 #include "System/TdfParser.h"
 #include "System/Input/KeyInput.h"
 #include "System/FileSystem/ArchiveScanner.h"
-#include "System/FileSystem/FileHandler.h"
+//// #include "System/FileSystem/FileHandler.h"
 #include "System/FileSystem/FileSystem.h"
 #include "System/FileSystem/VFSHandler.h"
 #include "System/LoadSave/DemoRecorder.h"
@@ -44,6 +44,7 @@
 #include "System/Net/RawPacket.h"
 #include "System/Net/UnpackPacket.h"
 #include "System/Platform/errorhandler.h"
+#include "System/Sync/SyncedPrimitiveBase.h"
 #include "lib/luasocket/src/restrictions.h"
 #ifdef SYNCDEBUG
 	#include "System/Sync/SyncDebugger.h"
@@ -57,48 +58,51 @@ CONFIG(bool, DemoFromDemo).defaultValue(false);
 
 CPreGame* pregame = NULL;
 
-CPreGame::CPreGame(boost::shared_ptr<const ClientSetup> setup)
-	: settings(setup)
+CPreGame::CPreGame(boost::shared_ptr<ClientSetup> setup)
+	: clientSetup(setup)
 	, savefile(NULL)
 	, timer(spring_gettime())
 	, wantDemo(true)
 {
-	net = new CNetProtocol();
+	assert(clientNet == NULL);
+
+	clientNet = new CNetProtocol();
 	activeController = this;
 
 #ifdef SYNCDEBUG
-	CSyncDebugger::GetInstance()->Initialize(settings->isHost, 64); //FIXME: add actual number of player
+	CSyncDebugger::GetInstance()->Initialize(clientSetup->isHost, 64); //FIXME: add actual number of player
 #endif
 
-	if (!settings->isHost) {
+	if (!clientSetup->isHost) {
 		//don't allow luasocket to connect to the host
-		LOG("Connecting to: %s:%i", settings->hostIP.c_str(), settings->hostPort);
-		luaSocketRestrictions->addRule(CLuaSocketRestrictions::UDP_CONNECT, settings->hostIP, settings->hostPort, false);
-		net->InitClient(settings->hostIP.c_str(), settings->hostPort, settings->myPlayerName, settings->myPasswd, SpringVersion::GetFull());
+		LOG("Connecting to: %s:%i", clientSetup->hostIP.c_str(), clientSetup->hostPort);
+		luaSocketRestrictions->addRule(CLuaSocketRestrictions::UDP_CONNECT, clientSetup->hostIP, clientSetup->hostPort, false);
+		clientNet->InitClient(clientSetup->hostIP.c_str(), clientSetup->hostPort, clientSetup->myPlayerName, clientSetup->myPasswd, SpringVersion::GetFull());
 	} else {
-		LOG("Hosting on: %s:%i", settings->hostIP.c_str(), settings->hostPort);
-		net->InitLocalClient();
+		LOG("Hosting on: %s:%i", clientSetup->hostIP.c_str(), clientSetup->hostPort);
+		clientNet->InitLocalClient();
 	}
 }
 
 
 CPreGame::~CPreGame()
 {
-	// don't delete infoconsole, its beeing reused by CGame
-	agui::gui->Draw(); // delete leftover gui elements (remove once the gui is drawn ingame)
+	// delete leftover elements (remove once the gui is drawn ingame)
+	// but do not delete infoconsole, it is reused by CGame
+	agui::gui->Draw();
 
 	pregame = NULL;
 }
 
 void CPreGame::LoadSetupscript(const std::string& script)
 {
-	assert(settings->isHost);
+	assert(clientSetup->isHost);
 	StartServer(script);
 }
 
 void CPreGame::LoadDemo(const std::string& demo)
 {
-	assert(settings->isHost);
+	assert(clientSetup->isHost);
 
 	if (!configHandler->GetBool("DemoFromDemo"))
 		wantDemo = false;
@@ -108,7 +112,7 @@ void CPreGame::LoadDemo(const std::string& demo)
 
 void CPreGame::LoadSavefile(const std::string& save)
 {
-	assert(settings->isHost);
+	assert(clientSetup->isHost);
 	savefile = ILoadSaveHandler::Create();
 	savefile->LoadGameStartInfo(save.c_str());
 
@@ -119,7 +123,7 @@ int CPreGame::KeyPressed(int k, bool isRepeat)
 {
 	if (k == SDLK_ESCAPE) {
 		if (KeyInput::GetKeyModState(KMOD_SHIFT)) {
-			LOG("User exited");
+			LOG("[%s] user exited", __FUNCTION__);
 			gu->globalQuit = true;
 		} else {
 			LOG("Use shift-esc to quit");
@@ -137,8 +141,8 @@ bool CPreGame::Draw()
 
 	font->Begin();
 
-	if (!net->Connected()) {
-		if (settings->isHost)
+	if (!clientNet->Connected()) {
+		if (clientSetup->isHost)
 			font->glFormat(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Waiting for server to start");
 		else
 			font->glFormat(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Connecting to server (%ds)", (spring_gettime() - timer).toSecsi());
@@ -146,9 +150,9 @@ bool CPreGame::Draw()
 		font->glPrint(0.5f, 0.48f, 2.0f, FONT_CENTER | FONT_SCALE | FONT_NORM, "Waiting for server response");
 	}
 
-	font->glFormat(0.60f, 0.40f, 1.0f, FONT_SCALE | FONT_NORM, "Connecting to:   %s", net->ConnectionStr().c_str());
+	font->glFormat(0.60f, 0.40f, 1.0f, FONT_SCALE | FONT_NORM, "Connecting to:   %s", clientNet->ConnectionStr().c_str());
 
-	font->glFormat(0.60f, 0.35f, 1.0f, FONT_SCALE | FONT_NORM, "User name: %s", settings->myPlayerName.c_str());
+	font->glFormat(0.60f, 0.35f, 1.0f, FONT_SCALE | FONT_NORM, "User name: %s", clientSetup->myPlayerName.c_str());
 
 	font->glFormat(0.5f,0.25f,0.8f,FONT_CENTER | FONT_SCALE | FONT_NORM, "Press SHIFT + ESC to quit");
 	// credits
@@ -165,52 +169,74 @@ bool CPreGame::Update()
 {
 	ENTER_SYNCED_CODE();
 	good_fpu_control_registers("CPreGame::Update");
-	net->Update();
+	clientNet->Update();
 	UpdateClientNet();
 	LEAVE_SYNCED_CODE();
 
 	return true;
 }
 
+void CPreGame::AddGameSetupArchivesToVFS(const CGameSetup* setup, bool mapOnly)
+{
+	LOG("[%s] using map: %s", __FUNCTION__, setup->mapName.c_str());
+	// Load Map archive
+	vfsHandler->AddArchiveWithDeps(setup->mapName, false);
+
+	if (mapOnly)
+		return;
+
+	// Load Mutators (if any)
+	for (const std::string& mut: setup->GetMutatorsCont()) {
+		LOG("[%s] using mutator: %s", __FUNCTION__, mut.c_str());
+		vfsHandler->AddArchiveWithDeps(mut, false);
+	}
+
+	// Load Game archive
+	vfsHandler->AddArchiveWithDeps(setup->modName, false);
+
+	modArchive = archiveScanner->ArchiveFromName(setup->modName);
+	LOG("[%s] using game: %s (archive: %s)", __FUNCTION__, setup->modName.c_str(), modArchive.c_str());
+}
 
 void CPreGame::StartServer(const std::string& setupscript)
 {
 	assert(!gameServer);
 	ScopedOnceTimer startserver("PreGame::StartServer");
-	GameData* startupData = new GameData();
-	CGameSetup* setup = new CGameSetup();
-	setup->Init(setupscript);
 
-	startupData->SetRandomSeed(static_cast<unsigned>(gu->RandInt()));
+	boost::shared_ptr<GameData> startGameData(new GameData());
+	boost::shared_ptr<CGameSetup> startGameSetup(new CGameSetup());
 
-	if (setup->mapName.empty()) {
+	startGameSetup->Init(setupscript);
+	startGameData->SetRandomSeed(static_cast<unsigned>(gu->RandInt()));
+
+	if (startGameSetup->mapName.empty()) {
 		throw content_error("No map selected in startscript");
 	}
 
-	if (setup->mapSeed != 0) {
-		CSimpleMapGenerator gen(setup);
+	if (startGameSetup->mapSeed != 0) {
+		CSimpleMapGenerator gen(startGameSetup.get());
 		gen.Generate();
 	}
 
 	// We must map the map into VFS this early, because server needs the start positions.
 	// Take care that MapInfo isn't loaded here, as map options aren't available to it yet.
-	vfsHandler->AddArchiveWithDeps(setup->mapName, false);
+	AddGameSetupArchivesToVFS(startGameSetup.get(), true);
 
 	// Loading the start positions executes the map's Lua.
 	// This means start positions can NOT be influenced by map options.
 	// (Which is OK, since unitsync does not have map options available either.)
-	setup->LoadStartPositions();
+	startGameSetup->LoadStartPositions();
 
-	const std::string& modArchive = archiveScanner->ArchiveFromName(setup->modName);
-	const std::string& mapArchive = archiveScanner->ArchiveFromName(setup->mapName);
-	startupData->SetModChecksum(archiveScanner->GetArchiveCompleteChecksum(modArchive));
-	startupData->SetMapChecksum(archiveScanner->GetArchiveCompleteChecksum(mapArchive));
+	const std::string& modArchive = archiveScanner->ArchiveFromName(startGameSetup->modName);
+	const std::string& mapArchive = archiveScanner->ArchiveFromName(startGameSetup->mapName);
+	startGameData->SetModChecksum(archiveScanner->GetArchiveCompleteChecksum(modArchive));
+	startGameData->SetMapChecksum(archiveScanner->GetArchiveCompleteChecksum(mapArchive));
 
 	good_fpu_control_registers("before CGameServer creation");
-	startupData->SetSetup(setup->gameSetupText);
-	gameServer = new CGameServer(settings->hostIP, settings->hostPort, startupData, setup);
-	delete startupData;
-	gameServer->AddLocalClient(settings->myPlayerName, SpringVersion::GetFull());
+	startGameData->SetSetupText(startGameSetup->setupText);
+	gameServer = new CGameServer(clientSetup, startGameData, startGameSetup);
+
+	gameServer->AddLocalClient(clientSetup->myPlayerName, SpringVersion::GetFull());
 	good_fpu_control_registers("after CGameServer creation");
 }
 
@@ -219,7 +245,7 @@ void CPreGame::UpdateClientNet()
 {
 	//FIXME move this code to a external file and move that to rts/Net/
 
-	if (net->CheckTimeout(0, true)) {
+	if (clientNet->CheckTimeout(0, true)) {
 		LOG_L(L_WARNING, "Server not reachable");
 		SetExitCode(1);
 		gu->globalQuit = true;
@@ -228,7 +254,7 @@ void CPreGame::UpdateClientNet()
 
 	boost::shared_ptr<const RawPacket> packet;
 
-	while ((packet = net->GetData(gs->frameNum))) {
+	while ((packet = clientNet->GetData(gs->frameNum))) {
 		const unsigned char* inbuf = packet->data;
 
 		if (packet->length <= 0) {
@@ -289,8 +315,6 @@ void CPreGame::UpdateClientNet()
 				// server first sends this to let us know about teams, allyteams
 				// etc. (not if we are joining mid-game as an extra player), see
 				// NETMSG_SETPLAYERNUM
-				if (gameSetup)
-					throw content_error("Duplicate game data received from server");
 				GameDataReceived(packet);
 				break;
 			}
@@ -326,99 +350,111 @@ void CPreGame::UpdateClientNet()
 	}
 }
 
+
+void CPreGame::StartServerForDemo(const std::string& demoName)
+{
+	TdfParser script((gameData->GetSetupText()).c_str(), (gameData->GetSetupText()).size());
+	TdfParser::TdfSection* tgame = script.GetRootSection()->sections["game"];
+
+	std::ostringstream moddedDemoScript;
+	std::string playerStr;
+
+	{
+		// server will always use a modified copy of this
+		assert(gameSetup != NULL);
+
+		// modify the demo's start-script so it can be used to watch the demo
+		tgame->AddPair("MapName", gameSetup->mapName);
+		tgame->AddPair("Gametype", gameSetup->modName);
+		tgame->AddPair("Demofile", demoName);
+		tgame->remove("OnlyLocal", false);
+		tgame->remove("HostIP", false);
+		tgame->remove("HostPort", false);
+		tgame->remove("AutohostPort", false);
+		tgame->remove("SourcePort", false);
+		//tgame->remove("IsHost", false);
+
+		for (std::map<std::string, TdfParser::TdfSection*>::iterator it = tgame->sections.begin(); it != tgame->sections.end(); ++it) {
+			if (it->first.size() > 6 && it->first.substr(0, 6) == "player") {
+				it->second->AddPair("isfromdemo", 1);
+			}
+		}
+
+		// add local spectator (and assert we didn't already have MAX_PLAYERS players)
+		for (int myPlayerNum = MAX_PLAYERS - 1; myPlayerNum >= 0; --myPlayerNum) {
+			string s = IntToString(myPlayerNum, "game\\player%i");
+			if (script.SectionExist(s)) {
+				playerStr = IntToString(myPlayerNum + 1, "player%i");
+				break;
+			}
+		}
+
+		assert(!playerStr.empty());
+
+		TdfParser::TdfSection* me = tgame->construct_subsection(playerStr);
+		me->AddPair("name", clientSetup->myPlayerName);
+		me->AddPair("spectator", 1);
+		tgame->AddPair("myplayername", clientSetup->myPlayerName);
+
+		// is this needed?
+		TdfParser::TdfSection* modopts = tgame->construct_subsection("MODOPTIONS");
+		modopts->remove("MaxSpeed", false);
+	}
+
+
+	script.print(moddedDemoScript);
+	gameData->SetSetupText(moddedDemoScript.str());
+
+	// create the server-private demo GameSetup containing the additional player
+	boost::shared_ptr<CGameSetup> demoGameSetup(new CGameSetup());
+
+	if (!demoGameSetup->Init(moddedDemoScript.str()))
+		throw content_error("Demo contains incorrect script");
+
+	LOG("[%s] starting GameServer", __FUNCTION__);
+	good_fpu_control_registers("before CGameServer creation");
+
+	gameServer = new CGameServer(clientSetup, gameData, demoGameSetup);
+	gameServer->AddLocalClient(clientSetup->myPlayerName, SpringVersion::GetFull());
+
+	good_fpu_control_registers("after CGameServer creation");
+	LOG("[%s] started GameServer", __FUNCTION__);
+}
+
 void CPreGame::ReadDataFromDemo(const std::string& demoName)
 {
 	ScopedOnceTimer startserver("PreGame::ReadDataFromDemo");
-	assert(!gameServer);
-	LOG("Pre-scanning demo file for game data...");
+	assert(gameServer == NULL);
+	LOG("[%s] pre-scanning demo file for game data...", __FUNCTION__);
 	CDemoReader scanner(demoName, 0);
 
 	boost::shared_ptr<const RawPacket> buf(scanner.GetData(static_cast<float>(FLT_MAX)));
 
 	while (buf) {
 		if (buf->data[0] == NETMSG_GAMEDATA) {
-			GameData* data = NULL;
 			try {
-				data = new GameData(boost::shared_ptr<const RawPacket>(buf));
+				gameData.reset(new GameData(boost::shared_ptr<const RawPacket>(buf)));
 			} catch (const netcode::UnpackPacketException& ex) {
 				throw content_error(std::string("Demo contains invalid GameData: ") + ex.what());
 			}
 
-			CGameSetup* demoScript = new CGameSetup();
-			if (!demoScript->Init(data->GetSetup())) {
+			if (CGameSetup::LoadReceivedScript(gameData->GetSetupText(), true)) {
+				StartServerForDemo(demoName);
+			} else {
 				throw content_error("Demo contains incorrect script");
 			}
 
-			// modify the startscriptscript so it can be used to watch the demo
-			TdfParser script(data->GetSetup().c_str(), data->GetSetup().size());
-			TdfParser::TdfSection* tgame = script.GetRootSection()->sections["game"];
-
-			tgame->AddPair("MapName", demoScript->mapName);
-			tgame->AddPair("Gametype", demoScript->modName);
-			tgame->AddPair("Demofile", demoName);
-			tgame->AddPair("OnlyLocal", 1);
-
-			for (std::map<std::string, TdfParser::TdfSection*>::iterator it = tgame->sections.begin(); it != tgame->sections.end(); ++it) {
-				if (it->first.size() > 6 && it->first.substr(0, 6) == "player") {
-					it->second->AddPair("isfromdemo", 1);
-				}
-			}
-
-			// add local spectator (and assert we didn't already have MAX_PLAYERS players)
-			int myPlayerNum;
-			string playerStr;
-			for (myPlayerNum = MAX_PLAYERS - 1; myPlayerNum >= 0; --myPlayerNum) {
-				char section[50];
-				sprintf(section, "game\\player%i", myPlayerNum);
-				string s(section);
-
-				if (script.SectionExist(s)) {
-					++myPlayerNum;
-					sprintf(section, "player%i", myPlayerNum);
-					playerStr = std::string(section);
-					break;
-				}
-			}
-
-			assert(!playerStr.empty());
-
-
-			TdfParser::TdfSection* me = tgame->construct_subsection(playerStr);
-			me->AddPair("name", settings->myPlayerName);
-			me->AddPair("spectator", 1);
-			tgame->AddPair("myplayername", settings->myPlayerName);
-
-			TdfParser::TdfSection* modopts = tgame->construct_subsection("MODOPTIONS");
-			modopts->AddPair("MaxSpeed", 20);
-
-			std::ostringstream buf;
-			script.print(buf);
-
-			data->SetSetup(buf.str());
-			CGameSetup* tempSetup = new CGameSetup();
-
-			if (!tempSetup->Init(buf.str())) {
-				throw content_error("Demo contains incorrect script");
-			}
-			LOG("Starting GameServer");
-			good_fpu_control_registers("before CGameServer creation");
-
-			gameServer = new CGameServer(settings->hostIP, settings->hostPort, data, tempSetup);
-			gameServer->AddLocalClient(settings->myPlayerName, SpringVersion::GetFull());
-			delete data;
-
-			good_fpu_control_registers("after CGameServer creation");
-			LOG("GameServer started");
 			break;
 		}
 
 		if (scanner.ReachedEnd()) {
 			throw content_error("End of demo reached and no game data found");
 		}
+
 		buf.reset(scanner.GetData(FLT_MAX));
 	}
 
-	assert(gameServer);
+	assert(gameServer != NULL);
 }
 
 void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> packet)
@@ -431,20 +467,18 @@ void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> pack
 		throw content_error(std::string("Server sent us invalid GameData: ") + ex.what());
 	}
 
-	CGameSetup* temp = new CGameSetup();
+	// for demos, ReadDataFromDemo precedes UpdateClientNet -> GameDataReceived
+	// this means gameSetup contains data from the original game but we need the
+	// modified version (cf StartServerForDemo) which the server already has that
+	// contains an extra player
+	if (gameSetup != NULL)
+		SafeDelete(gameSetup);
 
-	if (temp->Init(gameData->GetSetup())) {
-		if (settings->isHost) {
-			const std::string& setupTextStr = gameData->GetSetup();
-			std::fstream setupTextFile("_script.txt", std::ios::out);
-
-			setupTextFile.write(setupTextStr.c_str(), setupTextStr.size());
-			setupTextFile.close();
-		}
-		// set the global instance
-		gameSetup = temp;
+	if (CGameSetup::LoadReceivedScript(gameData->GetSetupText(), clientSetup->isHost)) {
+		assert(gameSetup != NULL);
 		gu->LoadFromSetup(gameSetup);
 		gs->LoadFromSetup(gameSetup);
+		// do we really need to do this so early?
 		CPlayer::UpdateControlledTeams();
 	} else {
 		throw content_error("Server sent us incorrect script");
@@ -465,23 +499,7 @@ void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> pack
 	}
 
 	// Load archives into VFS
-	{
-		// Load Map archive
-		LOG("Using map: %s", gameSetup->mapName.c_str());
-		vfsHandler->AddArchiveWithDeps(gameSetup->mapName, false);
-
-		// Load Mutators (if any)
-		for (const std::string& mut: gameSetup->GetMutatorsCont()) {
-			LOG("Using mutator: %s", mut.c_str());
-			vfsHandler->AddArchiveWithDeps(mut, false);
-		}
-
-		// Load Game archive
-		LOG("Using game: %s", gameSetup->modName.c_str());
-		vfsHandler->AddArchiveWithDeps(gameSetup->modName, false);
-		modArchive = archiveScanner->ArchiveFromName(gameSetup->modName);
-		LOG("Using game archive: %s", modArchive.c_str());
-	}
+	AddGameSetupArchivesToVFS(gameSetup, false);
 
 	// Check checksums of map & game
 	try {
@@ -495,13 +513,17 @@ void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> pack
 		LOG_L(L_WARNING, "Incompatible game-checksum: %s", ex.what());
 	}
 
-	if (net != NULL && wantDemo) {
-		assert(net->GetDemoRecorder() == NULL);
+	if (clientSetup->isHost && !gameSetup->recordDemo) { //script.txt allows to disable demo file recording (host only, used for menu)
+		wantDemo = false;
+	}
+
+	if (clientNet != NULL && wantDemo) {
+		assert(clientNet->GetDemoRecorder() == NULL);
 
 		CDemoRecorder* recorder = new CDemoRecorder(gameSetup->mapName, gameSetup->modName, false);
-		recorder->WriteSetupText(gameData->GetSetup());
-		recorder->SaveToDemo(packet->data, packet->length, net->GetPacketTime(gs->frameNum));
-		net->SetDemoRecorder(recorder);
+		recorder->WriteSetupText(gameData->GetSetupText());
+		recorder->SaveToDemo(packet->data, packet->length, clientNet->GetPacketTime(gs->frameNum));
+		clientNet->SetDemoRecorder(recorder);
 
 		LOG("Recording demo to: %s", (recorder->GetName()).c_str());
 	}
