@@ -58,50 +58,27 @@ static int GuiSoundSetTable(lua_State* L, const void* data);
 
 bool LuaWeaponDefs::PushEntries(lua_State* L)
 {
-	if (paramMap.empty()) {
-	  InitParamMap();
-	}
+	InitParamMap();
 
-	const map<string, int>& weaponMap = weaponDefHandler->weaponID;
-	map<string, int>::const_iterator wit;
-	for (wit = weaponMap.begin(); wit != weaponMap.end(); ++wit) {
-		const WeaponDef* wd = &weaponDefHandler->weaponDefs[wit->second];
-		if (wd == NULL) {
-	  	continue;
-		}
-		lua_pushnumber(L, wd->id);
-		lua_newtable(L); { // the proxy table
+	typedef int (*IndxFuncType)(lua_State*);
+	typedef int (*IterFuncType)(lua_State*);
+	typedef std::map<std::string, int> ObjectDefMapType;
 
-			lua_newtable(L); { // the metatable
+	const ObjectDefMapType& defsMap = weaponDefHandler->weaponID;
 
-				HSTR_PUSH(L, "__index");
-				lua_pushlightuserdata(L, (void*)wd);
-				lua_pushcclosure(L, WeaponDefIndex, 1);
-				lua_rawset(L, -3); // closure
+	const char* indxOpers[] = {"__index", "__newindex", "__metatable"};
+	const char* iterOpers[] = {"pairs", "next"};
 
-				HSTR_PUSH(L, "__newindex");
-				lua_pushlightuserdata(L, (void*)wd);
-				lua_pushcclosure(L, WeaponDefNewIndex, 1);
-				lua_rawset(L, -3);
+	const IndxFuncType indxFuncs[] = {&WeaponDefIndex, &WeaponDefNewIndex, &WeaponDefMetatable};
+	const IterFuncType iterFuncs[] = {&Pairs, &Next};
 
-				HSTR_PUSH(L, "__metatable");
-				lua_pushlightuserdata(L, (void*)wd);
-				lua_pushcclosure(L, WeaponDefMetatable, 1);
-				lua_rawset(L, -3);
-			}
+	for (auto it = defsMap.cbegin(); it != defsMap.cend(); ++it) {
+		const auto def = &weaponDefHandler->weaponDefs[it->second];
 
-			lua_setmetatable(L, -2);
-		}
+		if (def == NULL)
+			continue;
 
-		HSTR_PUSH(L, "pairs");
-		lua_pushcfunction(L, Pairs);
-		lua_rawset(L, -3);
-
-		HSTR_PUSH(L, "next");
-		lua_pushcfunction(L, Next);
-		lua_rawset(L, -3);
-
-		lua_rawset(L, -3); // proxy table into WeaponDefs
+		PushObjectDefProxyTable(L, indxOpers, iterOpers, indxFuncs, iterFuncs, def);
 	}
 
 	return true;
@@ -426,6 +403,8 @@ static int GuiSoundSetTable(lua_State* L, const void* data)
 
 static bool InitParamMap()
 {
+	paramMap.clear();
+
 	paramMap["next"]  = DataElement(READONLY_TYPE);
 	paramMap["pairs"] = DataElement(READONLY_TYPE);
 
@@ -473,6 +452,11 @@ static bool InitParamMap()
 	ADD_FLOAT("leadBonus", wd.leadBonus);
 	ADD_FLOAT("predictBoost", wd.predictBoost);
 	ADD_INT("highTrajectory", wd.highTrajectory);
+ 
+	ADD_FLOAT("dynDamageExp", wd.dynDamageExp);
+	ADD_FLOAT("dynDamageMin", wd.dynDamageMin);
+	ADD_FLOAT("dynDamageRange", wd.dynDamageRange);
+	ADD_BOOL("dynDamageInverted", wd.dynDamageInverted);
 
 	ADD_BOOL("noSelfDamage",  wd.noSelfDamage);
 	ADD_BOOL("impactOnly",    wd.impactOnly);
@@ -542,6 +526,7 @@ static bool InitParamMap()
 	ADD_FLOAT("dance",  wd.dance);
 
 	ADD_FLOAT("trajectoryHeight", wd.trajectoryHeight);
+	ADD_INT("flightTime", wd.flighttime);
 
 	ADD_BOOL("largeBeamLaser", wd.largeBeamLaser);
 	ADD_BOOL("laserHardStop", wd.laserHardStop);
