@@ -33,6 +33,7 @@ CR_REG_METADATA(CFeature, (
 	CR_MEMBER(inUpdateQue),
 	CR_MEMBER(resurrectProgress),
 	CR_MEMBER(reclaimLeft),
+	CR_MEMBER(resources),
 	CR_MEMBER(finalHeight),
 	CR_MEMBER(lastReclaim),
 	CR_MEMBER(drawQuad),
@@ -48,29 +49,30 @@ CR_REG_METADATA(CFeature, (
 ))
 
 
-CFeature::CFeature() : CSolidObject(),
-	defID(-1),
-	isRepairingBeforeResurrect(false),
-	isAtFinalHeight(false),
-	inUpdateQue(false),
-	resurrectProgress(0.0f),
-	reclaimLeft(1.0f),
-	finalHeight(0.0f),
-	lastReclaim(0),
-	drawQuad(-2),
-	fireTime(0),
-	smokeTime(0),
-
-	def(NULL),
-	udef(NULL),
-
-	myFire(NULL),
-	solidOnTop(NULL)
+CFeature::CFeature()
+: CSolidObject()
+, defID(-1)
+, isRepairingBeforeResurrect(false)
+, isAtFinalHeight(false)
+, inUpdateQue(false)
+, finalHeight(0.0f)
+, resurrectProgress(0.0f)
+, reclaimLeft(1.0f)
+, lastReclaim(0)
+, resources(0.0f, 1.0f)
+, drawQuad(-2)
+, fireTime(0)
+, smokeTime(0)
+, def(NULL)
+, udef(NULL)
+, myFire(NULL)
+, solidOnTop(NULL)
 {
 	crushable = true;
 	immobile = true;
 }
 
+
 CFeature::~CFeature()
 {
 	UnBlock();
@@ -90,6 +92,7 @@ CFeature::~CFeature()
 	}
 }
 
+
 void CFeature::PostLoad()
 {
 	def = featureHandler->GetFeatureDefByID(defID);
@@ -124,6 +127,7 @@ void CFeature::ChangeTeam(int newTeam)
 	}
 }
 
+
 bool CFeature::IsInLosForAllyTeam(int argAllyTeam) const
 {
 	if (alwaysVisible)
@@ -167,6 +171,8 @@ void CFeature::Initialize(const FeatureLoadParams& params)
 	mass = def->mass;
 	health = def->health;
 
+	resources = SResourcePack(def->metal, def->energy);
+
 	crushResistance = def->crushResistance;
 
 	xsize = ((buildFacing & 1) == 0) ? def->xsize : def->zsize;
@@ -269,7 +275,7 @@ bool CFeature::AddBuildPower(CUnit* builder, float amount)
 		// Work out how much that will cost
 		const float metalUse  = step * def->metal;
 		const float energyUse = step * def->energy;
-		const bool canExecRepair = (builderTeam->metal >= metalUse && builderTeam->energy >= energyUse);
+		const bool canExecRepair = (builderTeam->res.metal >= metalUse && builderTeam->res.energy >= energyUse);
 		const bool repairAllowed = !canExecRepair ? false : eventHandler.AllowFeatureBuildStep(builder, this, step);
 
 		if (repairAllowed) {
@@ -277,6 +283,10 @@ bool CFeature::AddBuildPower(CUnit* builder, float amount)
 			builder->UseEnergy(energyUse);
 
 			reclaimLeft += step;
+			resources.metal  += metalUse;
+			resources.energy += energyUse;
+			resources.metal  = std::min(resources.metal, def->metal);
+			resources.energy = std::min(resources.energy, def->energy);
 
 			if (reclaimLeft >= 1.0f) {
 				isRepairingBeforeResurrect = false; // They can start reclaiming it again if they so wish
@@ -290,8 +300,8 @@ bool CFeature::AddBuildPower(CUnit* builder, float amount)
 			return true;
 		} else {
 			// update the energy and metal required counts
-			teamHandler->Team(builder->team)->energyPull += energyUse;
-			teamHandler->Team(builder->team)->metalPull  += metalUse;
+			teamHandler->Team(builder->team)->resPull.energy += energyUse;
+			teamHandler->Team(builder->team)->resPull.metal  += metalUse;
 		}
 		return false;
 	} else {
@@ -315,28 +325,29 @@ bool CFeature::AddBuildPower(CUnit* builder, float amount)
 
 		// stop the last bit giving too much resource
 		const float reclaimLeftTemp = std::max(0.0f, reclaimLeft - step);
-
 		const float fractionReclaimed = oldReclaimLeft - reclaimLeftTemp;
-		const float metalFraction = def->metal * fractionReclaimed;
-		const float energyFraction = def->energy * fractionReclaimed;
+		const float metalFraction  = std::min(def->metal  * fractionReclaimed, resources.metal);
+		const float energyFraction = std::min(def->energy * fractionReclaimed, resources.energy);
 		const float energyUseScaled = metalFraction * modInfo.reclaimFeatureEnergyCostFactor;
 
-		if (!builder->UseEnergy(energyUseScaled)) {
-			teamHandler->Team(builder->team)->energyPull += energyUseScaled;
-			return false;
-		}
-
-		reclaimLeft = reclaimLeftTemp;
-
-		if ((modInfo.reclaimMethod == 1) && (reclaimLeft == 0.0f)) {
-			// All-at-end method
-			builder->AddMetal(def->metal, false);
-			builder->AddEnergy(def->energy, false);
+		SResourceOrder order;
+		order.quantum    = false;
+		order.overflow   = builder->harvestStorage.empty();
+		order.separate   = true;
+		order.use.energy = energyUseScaled;
+		if (reclaimLeftTemp == 0.0f) {
+			// always give remaining resources at the end
+			order.add.metal  = resources.metal;
+			order.add.energy = resources.energy;
 		}
 		else if (modInfo.reclaimMethod == 0) {
 			// Gradual reclaim
-			builder->AddMetal(metalFraction, false);
-			builder->AddEnergy(energyFraction, false);
+			order.add.metal  = metalFraction;
+			order.add.energy = energyFraction;
+		}
+		else if (modInfo.reclaimMethod == 1) {
+			// All-at-end method
+			// see `reclaimLeftTemp == 0.0f` case
 		}
 		else {
 			// Chunky reclaiming, work out how many chunk boundaries we crossed
@@ -345,19 +356,25 @@ bool CFeature::AddBuildPower(CUnit* builder, float amount)
 			const int newChunk = ChunkNumber(reclaimLeft);
 
 			if (oldChunk != newChunk) {
-				const float numChunks = (float)oldChunk - (float)newChunk;
-				builder->AddMetal(numChunks * def->metal * chunkSize, false);
-				builder->AddEnergy(numChunks * def->energy * chunkSize, false);
+				const float numChunks = oldChunk - newChunk;
+				order.add.metal  = std::min(numChunks * def->metal * chunkSize,  resources.metal);
+				order.add.energy = std::min(numChunks * def->energy * chunkSize, resources.energy);
 			}
 		}
 
+		if (!builder->IssueResourceOrder(&order)) {
+			return false;
+		}
+
+		resources  -= order.add;
+		reclaimLeft = reclaimLeftTemp;
+		lastReclaim = gs->frameNum;
+
 		// Has the reclaim finished?
 		if (reclaimLeft <= 0) {
 			featureHandler->DeleteFeature(this);
 			return false;
 		}
-
-		lastReclaim = gs->frameNum;
 		return true;
 	}
 
@@ -407,7 +424,8 @@ void CFeature::DoDamage(
 			// if a partially reclaimed corpse got blasted,
 			// ensure its wreck is not worth the full amount
 			// (which might be more than the amount remaining)
-			deathFeature->reclaimLeft = reclaimLeft;
+			deathFeature->resources.metal  *= (def->metal != 0.0f)  ? resources.metal  / def->metal  : 1.0f;
+			deathFeature->resources.energy *= (def->energy != 0.0f) ? resources.energy / def->energy : 1.0f;
 		}
 
 		featureHandler->DeleteFeature(this);
@@ -436,6 +454,7 @@ void CFeature::SetVelocity(const float3& v)
 	}
 }
 
+
 void CFeature::ForcedMove(const float3& newPos)
 {
 	// remove from managers
@@ -456,6 +475,7 @@ void CFeature::ForcedMove(const float3& newPos)
 	quadField->AddFeature(this);
 }
 
+
 void CFeature::ForcedSpin(const float3& newDir)
 {
 	// update local direction-vectors
@@ -569,6 +589,7 @@ bool CFeature::UpdatePosition()
 	return (IsMoving());
 }
 
+
 void CFeature::UpdateFinalHeight(bool useGroundHeight)
 {
 	if (isAtFinalHeight)
@@ -589,6 +610,7 @@ void CFeature::UpdateFinalHeight(bool useGroundHeight)
 	}
 }
 
+
 bool CFeature::Update()
 {
 	bool continueUpdating = UpdatePosition();
@@ -598,7 +620,7 @@ bool CFeature::Update()
 	continueUpdating |= (def->geoThermal);
 
 	if (smokeTime != 0) {
-		if (!((gs->frameNum + id) & 3) && projectileHandler->particleSaturation < 0.7f) {
+		if (!((gs->frameNum + id) & 3) && projectileHandler->GetParticleSaturation() < 0.7f) {
 			new CSmokeProjectile(NULL, midPos + gu->RandVector() * radius * 0.3f,
 				gu->RandVector() * 0.3f + UpVector, smokeTime / 6 + 20, 6, 0.4f, 0.5f);
 		}
@@ -628,6 +650,8 @@ void CFeature::StartFire()
 
 	myFire = new CFireProjectile(midPos, UpVector, 0, 300, 70, radius * 0.8f, 20.0f);
 }
+
+
 void CFeature::EmitGeoSmoke()
 {
 	if ((gs->frameNum + id % 5) % 5 == 0) {
@@ -659,7 +683,8 @@ void CFeature::EmitGeoSmoke()
 	const CUnit* u = dynamic_cast<CUnit*>(solidOnTop);
 
 	if (u == NULL || !u->unitDef->needGeo) {
-		if ((projectileHandler->particleSaturation < 0.7f) || (projectileHandler->particleSaturation < 1 && !(gs->frameNum & 3))) {
+		const float partSat = !(gs->frameNum & 3) ? 1.0f : 0.7f;
+		if (projectileHandler->GetParticleSaturation() < partSat) {
 			const float3 pPos = gu->RandVector() * 10.0f + float3(pos.x, pos.y - 10.0f, pos.z);
 			const float3 pSpeed = (gu->RandVector() * 0.5f) + (UpVector * 2.0f);
 
@@ -669,26 +694,5 @@ void CFeature::EmitGeoSmoke()
 }
 
 
-
-float CFeature::RemainingResource(float res) const
-{
-	// Gradual reclaim
-	if (modInfo.reclaimMethod == 0) {
-		return res * reclaimLeft;
-	}
-
-	// Old style - all reclaimed at the end
-	if (modInfo.reclaimMethod == 1) {
-		return res;
-	}
-
-	// Otherwise we are doing chunk reclaiming
-	float chunkSize = res / modInfo.reclaimMethod; // resource/no_chunks
-	float chunksLeft = math::ceil(reclaimLeft * modInfo.reclaimMethod);
-	return chunkSize * chunksLeft;
-}
-
-float CFeature::RemainingMetal() const { return RemainingResource(def->metal); }
-float CFeature::RemainingEnergy() const { return RemainingResource(def->energy); }
-int CFeature::ChunkNumber(float f) const { return int(math::ceil(f * modInfo.reclaimMethod)); }
+int CFeature::ChunkNumber(float f) { return int(math::ceil(f * modInfo.reclaimMethod)); }
 
