@@ -452,12 +452,6 @@ void CFeatureHandler::Update()
 
 	{
 		if (!toBeRemoved.empty()) {
-
-			eventHandler.DeleteSyncedObjects();
-
-			eventHandler.DeleteSyncedFeatures();
-
-
 			while (!toBeRemoved.empty()) {
 				CFeature* feature = GetFeature(toBeRemoved.back());
 				toBeRemoved.pop_back();
@@ -474,8 +468,6 @@ void CFeatureHandler::Update()
 				}
 			}
 		}
-
-		eventHandler.UpdateFeatures();
 	}
 
 	CFeatureSet::iterator fi = updateFeatures.begin();
@@ -518,17 +510,13 @@ void CFeatureHandler::SetFeatureUpdateable(CFeature* feature, bool updateable)
 
 void CFeatureHandler::TerrainChanged(int x1, int y1, int x2, int y2)
 {
-	const std::vector<int>& quads = quadField->GetQuadsRectangle(
+	const auto& quads = quadField->GetQuadsRectangle(
 		float3(x1 * SQUARE_SIZE, 0, y1 * SQUARE_SIZE),
 		float3(x2 * SQUARE_SIZE, 0, y2 * SQUARE_SIZE)
 	);
 
-	for (std::vector<int>::const_iterator qi = quads.begin(); qi != quads.end(); ++qi) {
-		std::list<CFeature*>::const_iterator fi;
-		const std::list<CFeature*>& features = quadField->GetQuad(*qi).features;
-
-		for (fi = features.begin(); fi != features.end(); ++fi) {
-			CFeature* feature = *fi;
+	for (const int qi: quads) {
+		for (CFeature* feature: quadField->GetQuad(qi).features) {
 			feature->UpdateFinalHeight(true);
 
 			// put this feature back in the update-queue
