@@ -67,7 +67,7 @@ CStarburstProjectile::CStarburstProjectile(const ProjectileParams& params): CWea
 	, areaOfEffect(0.0f)
 	, distanceToTravel(params.maxRange)
 
-	, uptime(0)
+	, uptime(params.upTime)
 	, age(0)
 
 	, oldSmoke(pos)
@@ -87,7 +87,11 @@ CStarburstProjectile::CStarburstProjectile(const ProjectileParams& params): CWea
 	if (weaponDef != NULL) {
 		maxSpeed = weaponDef->projectilespeed;
 		areaOfEffect = weaponDef->damageAreaOfEffect;
-		uptime = weaponDef->uptime * GAME_SPEED;
+		
+		// Default uptime is -1. Positive values override the weapondef.
+		if (uptime < 0) {
+			uptime = weaponDef->uptime * GAME_SPEED;
+		}
 
 		if (weaponDef->flighttime == 0) {
 			ttl = std::min(3000.0f, uptime + weaponDef->range / maxSpeed + 100);
@@ -121,20 +125,14 @@ CStarburstProjectile::CStarburstProjectile(const ProjectileParams& params): CWea
 #endif
 }
 
-void CStarburstProjectile::Detach()
+
+CStarburstProjectile::~CStarburstProjectile()
 {
-	// SYNCED
 	if (curCallback) {
 		// this is unsynced, but it prevents some callback crash on exit
 		curCallback->drawCallbacker = NULL;
 	}
 
-	CProjectile::Detach();
-}
-
-CStarburstProjectile::~CStarburstProjectile()
-{
-	// UNSYNCED
 	for (unsigned int a = 0; a < NUM_TRACER_PARTS; ++a) {
 		tracerParts[a].ageMods.clear();
 	}
@@ -144,7 +142,7 @@ CStarburstProjectile::~CStarburstProjectile()
 void CStarburstProjectile::Collision()
 {
 	if (weaponDef->visuals.smokeTrail) {
-		new CSmokeTrailProjectile(owner(), pos, oldSmoke, dir, oldSmokeDir, false, true, 7, SMOKE_TIME, 0.7f, drawTrail, 0, weaponDef->visuals.texture2);
+		new CSmokeTrailProjectile(owner(), pos, oldSmoke, dir, oldSmokeDir, false, true, 7, SMOKE_TIME, 0.7f, drawTrail, nullptr, weaponDef->visuals.texture2);
 	}
 
 	oldSmokeDir = dir;
@@ -155,7 +153,7 @@ void CStarburstProjectile::Collision()
 void CStarburstProjectile::Collision(CUnit* unit)
 {
 	if (weaponDef->visuals.smokeTrail) {
-		new CSmokeTrailProjectile(owner(), pos, oldSmoke, dir, oldSmokeDir, false, true, 7, SMOKE_TIME, 0.7f, drawTrail, 0, weaponDef->visuals.texture2);
+		new CSmokeTrailProjectile(owner(), pos, oldSmoke, dir, oldSmokeDir, false, true, 7, SMOKE_TIME, 0.7f, drawTrail, nullptr, weaponDef->visuals.texture2);
 	}
 
 	oldSmokeDir = dir;
@@ -181,12 +179,12 @@ void CStarburstProjectile::Update()
 	uptime--;
 	missileAge++;
 
-	if (target != NULL && owner() != NULL && weaponDef->tracks) {
+	if (target != NULL && allyteamID != -1 && weaponDef->tracks) {
 		targetPos = target->pos;
 		CUnit* u = dynamic_cast<CUnit*>(target);
 
 		if (u != NULL) {
-			targetPos = u->GetErrorPos(owner()->allyteam, true);
+			targetPos = u->GetErrorPos(allyteamID, true);
 		}
 	}
 
@@ -381,10 +379,10 @@ void CStarburstProjectile::Draw()
 				const float3 pos1 = CalcBeizer((float)a / curNumParts, pos, dirpos1, dirpos2, oldSmoke);
 
 				#define st projectileDrawer->smoketex[0]
-				va->AddVertexQTC(pos1 + ( camera->up + camera->right) * size, st->xstart, st->ystart, col);
-				va->AddVertexQTC(pos1 + ( camera->up - camera->right) * size, st->xend,   st->ystart, col);
-				va->AddVertexQTC(pos1 + (-camera->up - camera->right) * size, st->xend,   st->ystart, col);
-				va->AddVertexQTC(pos1 + (-camera->up + camera->right) * size, st->xstart, st->ystart, col);
+				va->AddVertexQTC(pos1 + ( camera->GetUp() + camera->GetRight()) * size, st->xstart, st->ystart, col);
+				va->AddVertexQTC(pos1 + ( camera->GetUp() - camera->GetRight()) * size, st->xend,   st->ystart, col);
+				va->AddVertexQTC(pos1 + (-camera->GetUp() - camera->GetRight()) * size, st->xend,   st->ystart, col);
+				va->AddVertexQTC(pos1 + (-camera->GetUp() + camera->GetRight()) * size, st->xstart, st->ystart, col);
 				#undef st
 			}
 		}
@@ -396,10 +394,10 @@ void CStarburstProjectile::Draw()
 void CStarburstProjectile::DrawCallback()
 {
 	inArray = true;
-
 	unsigned char col[4];
-
 	unsigned int part = curTracerPart;
+	const auto wt3 = weaponDef->visuals.texture3;
+	const auto wt1 = weaponDef->visuals.texture1;
 
 	for (unsigned int a = 0; a < NUM_TRACER_PARTS; ++a) {
 		const TracerPart* tracerPart = &tracerParts[part];
@@ -431,12 +429,10 @@ void CStarburstProjectile::DrawCallback()
 
 			const float drawsize = 1.0f + age2 * 0.8f * ageMod * 7;
 
-			#define wt3 weaponDef->visuals.texture3
-			va->AddVertexTC(interPos - camera->right * drawsize - camera->up * drawsize, wt3->xstart, wt3->ystart, col);
-			va->AddVertexTC(interPos + camera->right * drawsize - camera->up * drawsize, wt3->xend,   wt3->ystart, col);
-			va->AddVertexTC(interPos + camera->right * drawsize + camera->up * drawsize, wt3->xend,   wt3->yend,   col);
-			va->AddVertexTC(interPos - camera->right * drawsize + camera->up * drawsize, wt3->xstart, wt3->yend,   col);
-			#undef wt3
+			va->AddVertexTC(interPos - camera->GetRight() * drawsize - camera->GetUp() * drawsize, wt3->xstart, wt3->ystart, col);
+			va->AddVertexTC(interPos + camera->GetRight() * drawsize - camera->GetUp() * drawsize, wt3->xend,   wt3->ystart, col);
+			va->AddVertexTC(interPos + camera->GetRight() * drawsize + camera->GetUp() * drawsize, wt3->xend,   wt3->yend,   col);
+			va->AddVertexTC(interPos - camera->GetRight() * drawsize + camera->GetUp() * drawsize, wt3->xstart, wt3->yend,   col);
 		}
 
 		// unsigned, so LHS will wrap around to UINT_MAX
@@ -450,13 +446,10 @@ void CStarburstProjectile::DrawCallback()
 	col[3] =   1;
 
 	const float fsize = 25.0f;
-
-	#define wt1 weaponDef->visuals.texture1
-	va->AddVertexTC(drawPos - camera->right * fsize - camera->up * fsize, wt1->xstart, wt1->ystart, col);
-	va->AddVertexTC(drawPos + camera->right * fsize - camera->up * fsize, wt1->xend,   wt1->ystart, col);
-	va->AddVertexTC(drawPos + camera->right * fsize + camera->up * fsize, wt1->xend,   wt1->yend,   col);
-	va->AddVertexTC(drawPos - camera->right * fsize + camera->up * fsize, wt1->xstart, wt1->yend,   col);
-	#undef wt1
+	va->AddVertexTC(drawPos - camera->GetRight() * fsize - camera->GetUp() * fsize, wt1->xstart, wt1->ystart, col);
+	va->AddVertexTC(drawPos + camera->GetRight() * fsize - camera->GetUp() * fsize, wt1->xend,   wt1->ystart, col);
+	va->AddVertexTC(drawPos + camera->GetRight() * fsize + camera->GetUp() * fsize, wt1->xend,   wt1->yend,   col);
+	va->AddVertexTC(drawPos - camera->GetRight() * fsize + camera->GetUp() * fsize, wt1->xstart, wt1->yend,   col);
 }
 
 int CStarburstProjectile::ShieldRepulse(CPlasmaRepulser* shield, float3 shieldPos, float shieldForce, float shieldMaxSpeed)
@@ -481,3 +474,8 @@ int CStarburstProjectile::ShieldRepulse(CPlasmaRepulser* shield, float3 shieldPo
 
 	return 0;
 }
+
+int CStarburstProjectile::GetProjectilesCount() const
+{
+	return numParts + 1;
+}
