@@ -50,17 +50,24 @@
 
 std::vector< std::vector<int> > CUnitScript::teamVars;
 std::vector< std::vector<int> > CUnitScript::allyVars;
-int CUnitScript::globalVars[GLOBAL_VAR_COUNT] =  { 0 };
+int CUnitScript::globalVars[GLOBAL_VAR_COUNT] = { 0 };
 
 
 void CUnitScript::InitVars(int numTeams, int numAllyTeams)
 {
+	// clear all globals in case we reloaded
+	memset(&globalVars[0], 0, GLOBAL_VAR_COUNT * sizeof(globalVars[0]));
+
+	teamVars.clear();
 	teamVars.resize(numTeams, std::vector<int>());
+
+	allyVars.clear();
+	allyVars.resize(numAllyTeams, std::vector<int>());
+
 	for (int t = 0; t < numTeams; t++) {
 		teamVars[t].resize(TEAM_VAR_COUNT, 0);
 	}
-
-	allyVars.resize(numAllyTeams, std::vector<int>());
+	
 	for (int t = 0; t < numAllyTeams; t++) {
 		allyVars[t].resize(ALLY_VAR_COUNT, 0);
 	}
@@ -69,7 +76,6 @@ void CUnitScript::InitVars(int numTeams, int numAllyTeams)
 
 CUnitScript::CUnitScript(CUnit* unit, const std::vector<LocalModelPiece*>& pieces)
 	: unit(unit)
-	, yardOpen(false)
 	, busy(false)
 	, hasSetSFXOccupy(false)
 	, hasRockUnit(false)
@@ -340,7 +346,7 @@ void CUnitScript::RemoveAnim(AnimType type, const std::list<AnimInfo*>::iterator
 void CUnitScript::AddAnim(AnimType type, int piece, int axis, float speed, float dest, float accel)
 {
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber");
+		ShowUnitScriptError("Invalid piecenumber");
 		return;
 	}
 
@@ -380,6 +386,7 @@ void CUnitScript::AddAnim(AnimType type, int piece, int axis, float speed, float
 		default: {
 		} break;
 	}
+	assert(overrideType >= 0);
 
 	if (animInfoIt != anims[overrideType].end())
 		RemoveAnim(overrideType, animInfoIt);
@@ -468,7 +475,7 @@ void CUnitScript::Move(int piece, int axis, float speed, float destination)
 void CUnitScript::MoveNow(int piece, int axis, float destination)
 {
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber");
+		ShowUnitScriptError("Invalid piecenumber");
 		return;
 	}
 
@@ -486,7 +493,7 @@ void CUnitScript::MoveNow(int piece, int axis, float destination)
 void CUnitScript::TurnNow(int piece, int axis, float destination)
 {
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber");
+		ShowUnitScriptError("Invalid piecenumber");
 		return;
 	}
 
@@ -504,23 +511,22 @@ void CUnitScript::TurnNow(int piece, int axis, float destination)
 void CUnitScript::SetVisibility(int piece, bool visible)
 {
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber");
+		ShowUnitScriptError("Invalid piecenumber");
 		return;
 	}
 
 	pieces[piece]->scriptSetVisible = visible;
 }
 
-
 void CUnitScript::EmitSfx(int sfxType, int piece)
 {
 #ifndef _CONSOLE
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber for emit-sfx");
+		ShowUnitScriptError("Invalid piecenumber for emit-sfx");
 		return;
 	}
 
-	if (projectileHandler->particleSaturation > 1.0f && sfxType < SFX_CEG) {
+	if (projectileHandler->GetParticleSaturation() > 1.0f && sfxType < SFX_CEG) {
 		// skip adding (unsynced!) particles when we have too many
 		return;
 	}
@@ -536,7 +542,7 @@ void CUnitScript::EmitSfx(int sfxType, int piece)
 	float3 relDir = UpVector;
 
 	if (!GetEmitDirPos(piece, relPos, relDir)) {
-		ShowScriptError("emit-sfx: GetEmitDirPos failed");
+		ShowUnitScriptError("emit-sfx: GetEmitDirPos failed");
 		return;
 	}
 
@@ -633,26 +639,24 @@ void CUnitScript::EmitSfx(int sfxType, int piece)
 			else if (sfxType & SFX_FIRE_WEAPON) {
 				// make a weapon fire from the piece
 				const unsigned index = sfxType - SFX_FIRE_WEAPON;
-				if (index >= unit->weapons.size() || unit->weapons[index] == NULL) {
-					ShowScriptError("Invalid weapon index for emit-sfx");
+				if (index >= unit->weapons.size()) {
+					ShowUnitScriptError("Invalid weapon index for emit-sfx");
 					break;
 				}
-
-				CWeapon* weapon = unit->weapons[index];
-
-				const float3 targetPos = weapon->targetPos;
-				const float3 weaponMuzzlePos = weapon->weaponMuzzlePos;
-
-				weapon->targetPos = pos + dir;
-				weapon->weaponMuzzlePos = pos;
-				weapon->Fire(true);
-				weapon->weaponMuzzlePos = weaponMuzzlePos;
-				weapon->targetPos = targetPos;
+				CWeapon* w = unit->weapons[index];
+				const SWeaponTarget origTarget = w->GetCurrentTarget();
+				const float3 origWeaponMuzzlePos = w->weaponMuzzlePos;
+				w->SetAttackTarget(SWeaponTarget(pos + dir));
+				w->weaponMuzzlePos = pos;
+				w->Fire(true);
+				w->weaponMuzzlePos = origWeaponMuzzlePos;
+				bool origRestored = w->Attack(origTarget);
+				assert(origRestored);
 			}
 			else if (sfxType & SFX_DETONATE_WEAPON) {
 				const unsigned index = sfxType - SFX_DETONATE_WEAPON;
-				if (index >= unit->weapons.size() || unit->weapons[index] == NULL) {
-					ShowScriptError("Invalid weapon index for emit-sfx");
+				if (index >= unit->weapons.size()) {
+					ShowUnitScriptError("Invalid weapon index for emit-sfx");
 					break;
 				}
 
@@ -692,7 +696,7 @@ void CUnitScript::AttachUnit(int piece, int u)
 {
 	// -1 is valid, indicates that the unit should be hidden
 	if ((piece >= 0) && (!PieceExists(piece))) {
-		ShowScriptError("Invalid piecenumber for attach");
+		ShowUnitScriptError("Invalid piecenumber for attach");
 		return;
 	}
 
@@ -751,7 +755,7 @@ bool CUnitScript::AddAnimListener(AnimType type, int piece, int axis, IAnimListe
 void CUnitScript::Explode(int piece, int flags)
 {
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber for explode");
+		ShowUnitScriptError("Invalid piecenumber for explode");
 		return;
 	}
 
@@ -804,11 +808,14 @@ void CUnitScript::Explode(int piece, int flags)
 	// projectiles that don't fall could live forever
 	int newflags = PF_Fall;
 
+	const float partSat = projectileHandler->GetParticleSaturation();
+
 	if (flags & PF_Explode) { newflags |= PF_Explode; }
 	// if (flags & PF_Fall) { newflags |=  PF_Fall; }
-	if ((flags & PF_Smoke) && projectileHandler->particleSaturation < 1.0f) { newflags |= PF_Smoke; }
-	if ((flags & PF_Fire) && projectileHandler->particleSaturation < 0.95f) { newflags |= PF_Fire; }
+	if ((flags & PF_Smoke) && partSat < 1.0f) { newflags |= PF_Smoke; }
+	if ((flags & PF_Fire) && partSat < 0.95f) { newflags |= PF_Fire; }
 	if (flags & PF_NoCEGTrail) { newflags |= PF_NoCEGTrail; }
+	if (flags & PF_Recursive) { newflags |= PF_Recursive; }
 
 	new CPieceProjectile(unit, pieces[piece], absPos, explSpeed, newflags, 0.5f);
 #endif
@@ -819,7 +826,7 @@ void CUnitScript::Shatter(int piece, const float3& pos, const float3& speed)
 {
 	const LocalModelPiece* lmp = pieces[piece];
 	const S3DModelPiece* omp = lmp->original;
-	const float pieceChance = 1.0f - (projectileHandler->currentParticles - (projectileHandler->maxParticles - 2000)) / 2000.0f;
+	const float pieceChance = 1.0f - (projectileHandler->GetCurrentParticles() - (projectileHandler->maxParticles - 2000)) / 2000.0f;
 
 	if (pieceChance > 0.0f) {
 		omp->Shatter(pieceChance, unit->model->textureType, unit->team, pos, speed);
@@ -830,7 +837,7 @@ void CUnitScript::Shatter(int piece, const float3& pos, const float3& speed)
 void CUnitScript::ShowFlare(int piece)
 {
 	if (!PieceExists(piece)) {
-		ShowScriptError("Invalid piecenumber for show(flare)");
+		ShowUnitScriptError("Invalid piecenumber for show(flare)");
 		return;
 	}
 #ifndef _CONSOLE
@@ -851,7 +858,7 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 {
 	// may happen in case one uses Spring.GetUnitCOBValue (Lua) on a unit with CNullUnitScript
 	if (!unit) {
-		ShowScriptError("Error: no unit (in GetUnitVal)");
+		ShowUnitScriptError("no unit (in GetUnitVal)");
 		return 0;
 	}
 
@@ -894,7 +901,7 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 		break;
 	case PIECE_XZ: {
 		if (!PieceExists(p1)) {
-			ShowScriptError("Invalid piecenumber for get piece_xz");
+			ShowUnitScriptError("Invalid piecenumber for get piece_xz");
 			break;
 		}
 		const float3 relPos = GetPiecePos(p1);
@@ -903,7 +910,7 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 	}
 	case PIECE_Y: {
 		if (!PieceExists(p1)) {
-			ShowScriptError("Invalid piecenumber for get piece_y");
+			ShowUnitScriptError("Invalid piecenumber for get piece_y");
 			break;
 		}
 		const float3 relPos = GetPiecePos(p1);
@@ -959,17 +966,11 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 		return int((1.0f - unit->buildProgress) * 100);
 
 	case YARD_OPEN:
-		if (yardOpen)
-			return 1;
-		else
-			return 0;
+		return (unit->yardOpen) ? 1 : 0;
 	case BUGGER_OFF:
 		break;
 	case ARMORED:
-		if (unit->armoredState)
-			return 1;
-		else
-			return 0;
+		return (unit->armoredState) ? 1 : 0;
 	case VETERAN_LEVEL:
 		return int(100 * unit->experience);
 	case CURRENT_SPEED:
@@ -998,11 +999,9 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 	}
 	case UNIT_BUILD_PERCENT_LEFT: {
 		const CUnit* u = unitHandler->GetUnit(p1);
-
 		if (u != NULL) {
 			return int((1.0f - u->buildProgress) * 100);
 		}
-
 		return 0;
 	}
 	case MAX_SPEED: {
@@ -1033,22 +1032,18 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 		if (u != NULL) {
 			return u->heading;
 		}
-
 		return -1;
 	}
 	case TARGET_ID: {
-		if (unit->weapons[p1 - 1]) {
-			const CWeapon* weapon = unit->weapons[p1 - 1];
-			const TargetType tType = weapon->targetType;
-
-			if (tType == Target_Unit)
-				return unit->weapons[p1 - 1]->targetUnit->id;
-			else if (tType == Target_None)
-				return -1;
-			else if (tType == Target_Pos)
-				return -2;
-			else // Target_Intercept
-				return -3;
+		if (size_t(p1 - 1) < unit->weapons.size()) {
+			const CWeapon* w = unit->weapons[p1 - 1];
+			auto curTarget = w->GetCurrentTarget();
+			switch (curTarget.type) {
+				case Target_Unit:      return curTarget.unit->id;
+				case Target_None:      return -1;
+				case Target_Pos:       return -2;
+				case Target_Intercept: return -3;
+			}
 		}
 		return -4; // weapon does not exist
 	}
@@ -1102,9 +1097,6 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 	}
 	case CRASHING:
 		return !!unit->IsCrashing();
-	case ALPHA_THRESHOLD: {
-		return int(unit->alphaThreshold * 255);
-	}
 
 	case COB_ID: {
 		if (p1 <= 0) {
@@ -1179,7 +1171,7 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 		//! if targetID is 0, just sets weapon->haveUserTarget
 		//! to false (and targetType to None) without attacking
 		CUnit* target = (targetID > 0)? unitHandler->GetUnit(targetID): NULL;
-		return (weapon->AttackUnit(target, userTarget) ? 1 : 0);
+		return (weapon->Attack(SWeaponTarget(target, userTarget)) ? 1 : 0);
 	}
 	case SET_WEAPON_GROUND_TARGET: {
 		const int weaponID = p1 - 1;
@@ -1193,7 +1185,7 @@ int CUnitScript::GetUnitVal(int val, int p1, int p2, int p3, int p4)
 		CWeapon* weapon = unit->weapons[weaponID];
 		if (weapon == NULL) { return 0; }
 
-		return weapon->AttackGround(pos, userTarget) ? 1 : 0;
+		return weapon->Attack(SWeaponTarget(pos, userTarget)) ? 1 : 0;
 	}
 	case MIN:
 		return std::min(p1, p2);
@@ -1395,7 +1387,7 @@ void CUnitScript::SetUnitVal(int val, int param)
 {
 	// may happen in case one uses Spring.SetUnitCOBValue (Lua) on a unit with CNullUnitScript
 	if (!unit) {
-		ShowScriptError("Error: no unit (in SetUnitVal)");
+		ShowUnitScriptError("Error: no unit (in SetUnitVal)");
 		return;
 	}
 
@@ -1486,12 +1478,10 @@ void CUnitScript::SetUnitVal(int val, int param)
 				if (param == 0) {
 					if (groundBlockingObjectMap->CanCloseYard(unit)) {
 						groundBlockingObjectMap->CloseBlockingYard(unit);
-						yardOpen = false;
 					}
 				} else {
 					if (groundBlockingObjectMap->CanOpenYard(unit)) {
 						groundBlockingObjectMap->OpenBlockingYard(unit);
-						yardOpen = true;
 					}
 				}
 			}
@@ -1605,10 +1595,6 @@ void CUnitScript::SetUnitVal(int val, int param)
 			unit->weapons[param]->avoidTarget = true;
 			break;
 		}
-		case ALPHA_THRESHOLD: {
-			unit->alphaThreshold = param / 255.0f;
-			break;
-		}
 		case CEG_DAMAGE: {
 			unit->cegDamage = param;
 			break;
@@ -1675,3 +1661,8 @@ int CUnitScript::ModelToScript(int lmodelPieceNum) const {
 	return (lmp->GetScriptPieceIndex());
 }
 
+void CUnitScript::ShowUnitScriptError(const std::string& error)
+{
+	ShowScriptError(error + std::string(" ") + IntToString(unit->id) + std::string(" of type ") + unit->unitDef->name);
+}
+
