@@ -81,7 +81,8 @@ UnitDef::UnitDef()
 	, tidalGenerator(0.0f)
 	, metalStorage(0.0f)
 	, energyStorage(0.0f)
-	, harvestStorage(0.0f)
+	, harvestMetalStorage(0.0f)
+	, harvestEnergyStorage(0.0f)
 	, autoHeal(0.0f)
 	, idleAutoHeal(0.0f)
 	, idleTime(0)
@@ -196,6 +197,7 @@ UnitDef::UnitDef()
 	, maxAileron(0.0f)
 	, maxElevator(0.0f)
 	, maxRudder(0.0f)
+	, attackSafetyDistance(0.0f)
 	, crashDrag(0.0f)
 	, loadingRadius(0.0f)
 	, unloadSpread(0.0f)
@@ -270,7 +272,8 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 
 	metalStorage  = udTable.GetFloat("metalStorage",  0.0f);
 	energyStorage = udTable.GetFloat("energyStorage", 0.0f);
-	harvestStorage = udTable.GetFloat("harvestStorage", 0.0f);
+	harvestMetalStorage  = udTable.GetFloat("harvestMetalStorage", udTable.GetFloat("harvestStorage", 0.0f));
+	harvestEnergyStorage = udTable.GetFloat("harvestEnergyStorage", 0.0f);
 
 	extractsMetal  = udTable.GetFloat("extractsMetal",  0.0f);
 	windGenerator  = udTable.GetFloat("windGenerator",  0.0f);
@@ -490,6 +493,8 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	maxElevator = udTable.GetFloat("maxElevator", 0.01f);  // turn speed around pitch axis
 	maxRudder   = udTable.GetFloat("maxRudder",   0.004f); // turn speed around yaw axis
 
+	attackSafetyDistance = udTable.GetFloat("attackSafetyDistance", 0.0f); // fighters abort dive toward target if within this distance and climb back to normal altitude
+
 	maxFuel = udTable.GetFloat("maxFuel", 0.0f); //max flight time in seconds before aircraft must return to base
 	refuelTime = udTable.GetFloat("refuelTime", 5.0f);
 	minAirBasePower = udTable.GetFloat("minAirBasePower", 0.0f);
